#!/bin/bash

# Author(s):
# Sascha Wilde <wilde@intevation.de>

ME=`basename "$0"`
BASEDIR=`dirname "$0"`
TRANSFERDIR_BASE="/home/mpuls/jmd"

usage()
{
  cat <<EOF
$ME [OPTION]... [PATH]

Update sqlite transfer based on jmd version in PATH.
Default PATH is: $path

Options:
  -b, --base       update base case management databases
  -s, --struktur   update struktur databases
  -c, --coaches    update respekt coaches databases
      --help       display this help and exit

EOF
}

fatal()
{
  echo >&2 "$1"
  exit 23
}


create_suffixs_array()
{
  declare -g suffixes=()
  [ $base -eq 1 ] && suffixes+=("")
  [ $struktur -eq 1 ] && suffixes+=("-struktur")
  [ $coaches -eq 1 ] && suffixes+=("-coaches")
}

create_dbs()
{
  for s in "${suffixes[@]}" ; do
    p="${path}/transfer${s}"
    [ -d "$p" ] || fatal "directory ${p} does not exist"
    cd "$p"
    su postgres -c ./update-template-db
  done
}

copy_dbs()
{
  for s in "${suffixes[@]}" ; do
    srcd="${path}/transfer${s}"
    dstd="${TRANSFERDIR_BASE}/transfer${s}"
    [ -d "$srcd" ] || fatal "source dir ${srcd} does not exist"
    [ -d "$dstd" ] || fatal "destination dir ${dstd} does not exist"
    for cluster in 1000 2000 3000 4000; do
      t="template${s}${cluster}.db"
      # backup old target:
      su - mpuls -c "mv ${dstd}/${t} ${dstd}/${t}.sik"
      su - mpuls -c "cp ${srcd}/${t} ${dstd}/${t}"
    done
  done
}


# Defaults:

path=/var/lib/postgresql/jmd/releases/jmd-current/application-db
base=0
struktur=0
coaches=0


# Parse options:

OPTS=`getopt \
      -l help,base,struktur,coaches \
      -o bsc -n "$ME" -- "$@"`
[ $? -eq 0 ] || { usage ; exit 1 ; }

eval set -- "$OPTS"

while true ; do
  case "$1" in
    --base|-b)
      base=1
      shift 1
      ;;
    --struktur|-s)
      struktur=1
      shift 1
      ;;
    --coaches|-c)
      coaches=1
      shift 1
      ;;
    --help)
      { usage ; exit 0 ; }
      ;;
    --)
      shift
      break
      ;;
  esac
done

case "$#" in
  0)
    # keep default path
    ;;
  1)
    path="$1"
    ;;
  *)
    usage
    exit 23
    ;;
esac

# Main ------------------------------------------------------------
create_suffixs_array
create_dbs
copy_dbs
