#!/bin/sh
set -e
set -x

get_port ()
# Args: $1 = ID
# Return DB Port for given ID
{
  case $1 in
    3*)
      echo 5432
      return
      ;;
    2*)
      echo 5433
      return
      ;;
    4*)
      echo 5434
      return
      ;;
    1*)
      echo 5435
      return
      ;;
    *)
      fatal "Could not determine db port for id $1"
      ;;
  esac
}

FKZBASE="$1"
TYPE=jmd
DBHOST=localhost
DBPORT=`get_port $FKZBASE`
EXPORTHOST=jmd-auswertung.intevation.de
EXPORTUSER=mpuls
EXPORTCOMMAND=/home/mpuls/${TYPE}/releases/current/evaluation-transfer/export-sqlite.py
TEMPLATEDIR=/home/mpuls/${TYPE}/transfer
DATADIR=${TEMPLATEDIR}/data/${FKZBASE}
TEMPLATEDB=$TEMPLATEDIR/template${FKZBASE}.db

DATE=`date +"%Y-%m-%d"`
EXPORTFILE=$DATADIR/${TYPE}-$DATE.db
cp $TEMPLATEDB $EXPORTFILE
chmod 666 $EXPORTFILE
echo "Exporting databases"
sudo -u postgres $EXPORTCOMMAND -o $EXPORTFILE -p $DBPORT \
                     -r "rg_institution,rg_kontaktdaten,rg_kontaktdaten_sorgeberechtigte,rg_kinder,rg_biographie2,rg_weitere_sprache,rg_ziele" \
                     --blacklist="id,uuid_id,datum_ea_ende,datum_ea_ende_bak"
bzip2 $EXPORTFILE
chmod 640 $EXPORTFILE.bz2

echo "Transfering databases"
scp -qp ${EXPORTFILE}.bz2 ${EXPORTUSER}@${EXPORTHOST}:${DATADIR}
echo "Done :)"
