-- Short-term fix for triggers that use ka_logbuch_tbl_view.typ.
-- See mpuls/issue6907
--
-- This is only an ad-hoc fix that solves the problem that no statistics
-- can be created. It doesn't solve the problem that the "typ" column
-- doesn't exist in (probably only older) struktur-databases.
--
-- The fix is to remove the assignment to "typ" from
-- master_ds_created_logbook.

-- Update master_ds_created_logbook
--
-- Since it's a trigger functions, drop the trigger first and re-create
-- it afterwards.

DROP TRIGGER master_ds_created_logbook_trigger ON master_tbl;


CREATE OR REPLACE FUNCTION master_ds_created_logbook() RETURNS trigger AS $master_ds_created_logbook$
DECLARE
    lid integer;
BEGIN
    SELECT INTO lid create_logbuch_ds(NEW.id, NULL);
    UPDATE ka_logbuch_tbl_view
        SET art = 0,
            notiz = notiz_from_plaintext('Fallakte angelegt.')
    WHERE
        id = lid;
    RETURN NEW;
END;
$master_ds_created_logbook$ LANGUAGE plpgsql;

CREATE TRIGGER master_ds_created_logbook_trigger AFTER
    INSERT ON master_tbl
    FOR EACH ROW EXECUTE PROCEDURE master_ds_created_logbook();
