# -*- coding: utf-8 -*-
#
# Copyright 2022 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
# Bernhard Herzog <bernhard.herzog@intevation.de>

from mpulsweb.lib.navigation import NavigationTreeBuilder


class IQTreeBuilder(NavigationTreeBuilder):

    """NavigationTreeBuilder that hides the JMD-iQ specific parts"""

    jmd_iq_value = None

    def has_jmd_iq(self, nc):
        if self.jmd_iq_value is not None:
            return self.jmd_iq_value
        tree = nc.getInstanceTree()
        for item in tree.getAllItemsForName("jmd_iq"):
            self.jmd_iq_value = item.getValue() == 1
        return self.jmd_iq_value

    def build(self, nc, parent):
        if not self.has_jmd_iq(nc) and nc.getMeta().getName() == "rg_jmd_iq":
            return None
        else:
            return super(IQTreeBuilder, self).build(nc, parent)
