# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

import logging

from pylons import url
from pylons.controllers.util import redirect, abort

from mpulsweb.lib.translation import _

import paste

from mpulsweb.lib.base import BaseController, c, h, render, \
    request, abort_unless_POST
from mpulsweb.lib.security import checkRole
from mpulsweb.model.document import Document, GlobalDocumentAPI, \
    DocumentNotFoundException
from mpulsweb.lib.formedforms import DocumentNewForm, DocumentRenameForm


log = logging.getLogger(__name__)


global_document_new_form = DocumentNewForm("document", "global_upload_action")

class RenameForm(DocumentRenameForm):

    def get_api(self, instance):
        return GlobalDocumentAPI()


global_document_rename_form = RenameForm("document", "global_rename_action")


class DocumentController(BaseController):

    @checkRole(('admin', 'cm'))
    def index(self):
        redirect(url(controller='document', action='globalOverview'))

    @checkRole(('admin', 'cm'))
    def globalOverview(self):
        c.files = GlobalDocumentAPI().list_db()
        return render('/documents/global_overview.mako')

    def _render_new_global_document(self, instance, errors):
        c.rendered_form = global_document_new_form.render(instance, errors)
        return render('/documents/global_upload.mako')

    @checkRole('admin')
    def globalUpload(self):
        instance = global_document_new_form.create_instance()
        return self._render_new_global_document(instance, {})

    @checkRole('admin')
    def global_upload_action(self):
        instance, errors = global_document_new_form.validate_request(
                                                                request.params)

        if errors:
            return self._render_new_global_document(instance, errors)

        GlobalDocumentAPI().upload_documents_transaction(instance.get("file"))
        c.success_text = _(u"Document added!")
        return self.globalOverview()

    @checkRole(('admin', 'cm'))
    def globalShow(self, id):
        id = self._checkInt(id)
        try:
            document = self._load_document(id, load_contents_immediately=True)
        except DocumentNotFoundException:
            # the document could not be found, so we should send a 404
            # response to the client.
            abort(404)

        data = paste.fileapp.DataApp(document.data,
                                     content_type=document.mime,
                                     cache_control='must-revalidate',
                                     pragma='public')
        return data(request.environ, self.start_response)

    def _load_document(self, id, load_contents_immediately):
        document = Document(id)
        document.load(load_contents_immediately=load_contents_immediately)
        return document

    @checkRole('admin')
    def globalDelete(self, id, confirmed='0'):
        id = self._checkInt(id)
        confirmed = self._checkBool(confirmed)
        if confirmed == 1:
            abort_unless_POST()
            try:
                GlobalDocumentAPI().delete_transaction(id)
            except Exception, e:
                log.exception(e)
                return self.globalOverview()
            c.success_text = (_(u"Document deleted!"))
            return self.globalOverview()
        else:
            doc = self._load_document(id, load_contents_immediately=False)
            name = doc.getName()
            if not name:
                name = ""

            c.context = "../main.mako"
            c.dialog_title = _(u"Delete document?")
            c.dialog_text = _(
                u"Do you want to delete the document '%s' and "
                u"irrevocably lose the data?") % name
            c.url_yes = h.url(controller='document',
                              action='globalDelete', id=id, confirmed=1)
            c.url_no = h.url(controller='document',
                             action='globalOverview')
            return render('/documents/dialogs/confirm.mako')


    def _render_global_rename_form(self, instance, errors):
        c.rendered_form = global_document_rename_form.render(instance, errors)
        return render('/documents/global_rename.mako')

    @checkRole('admin')
    def globalRename(self, id):
        doc = self._load_document(self._checkInt(id),
                                  load_contents_immediately=False)
        instance = global_document_rename_form.from_object(doc)
        return self._render_global_rename_form(instance, {})

    @checkRole('admin')
    def global_rename_action(self):
        instance, errors = global_document_rename_form.validate_request(
                                                                request.params)

        if errors:
            return self._render_global_rename_form(instance, errors)

        id = instance.get("id")
        GlobalDocumentAPI().rename_transaction(id, instance.get("name"))
        c.success_text = (_(u"Document renamed!"))
        return self.globalOverview()

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
