# -*- coding: utf-8 -*-

import logging

from pylons import tmpl_context as c, app_globals as g
from pylons.controllers.util import abort, redirect

from mpulsweb.lib.translation import _
from mpulsweb.lib.base import BaseController, render, session, \
     abort_unless_POST, restrict_to_POST
import mpulsweb.lib.helpers as h


log = logging.getLogger(__name__)


class RepeatgroupController(BaseController):

    def delete(self, id, confirmed='0'):
        """Delete rg"""
        log.debug("deleting repeat group %s" % id)
        case = self._load_case_for_node_id(id)
        instance_tree = case.get_formed_instance_tree()
        rg = instance_tree.getItem(id)
        if not rg:
            abort(404, _('Repeat group not found.'))
        if confirmed == '1':
            abort_unless_POST()
            try:
                rg.getParent().deleteInstance(rg)
            except:
                log.exception("Error trying to delete repeat group %r:", id)
                c.status_msg = h.status_message(u"Fehler beim Löschen"
                                                u" des Datensatzes.")

            # Build new Navigation
            navigation = g.get_navigation(case.id,
                                          session['navigation.openfolders'])
            session['navigation.tree'] = navigation
            session.save()

            # Want the id of the rg without the delete rg instance.
            page_id = session.get('navigation.selectedpage')
            id_split = id.split(':')
            rg_id = "%s:%s" % (id_split[0], id_split[2])
            rg_url = h.url(controller='formularpage', action='select',
                           id=page_id, _anchor=rg_id)

            phase_changed = self._handle_automatic_phase_change(case, rg_url)
            if phase_changed:
                return phase_changed
            redirect(rg_url)
        else:
            c.confirm_for = _(u"Delete dataset?")
            c.question = _(u"Do you want to delete this dataset and\n"
                           u"irrevocably lose the data?")
            c.url_yes = h.url(controller='repeatgroup', action='delete',
                              id=id, confirmed=1)
            c.url_no = h.url(controller='formularpage', action='select',
                             id=session['navigation.selectedpage'])
            return render('/repeatgroup/dialogs/confirm_delete_rg_entry.mako')

    @restrict_to_POST
    def new(self, id):
        """New rg"""
        log.debug("creating new repeat group %s" % id)
        case = self._load_case_for_node_id(id)
        rg_url = self._add_rg(case, id)
        phase_changed = self._handle_automatic_phase_change(case, rg_url)
        if phase_changed:
            return phase_changed
        redirect(rg_url)

