function getFormFieldElem(fieldname, pseudoclass) {
  if (typeof pseudoclass == "undefined") {
    pseudoclass = ""
  }
  return document.querySelector("#formularpage *[name=\"" + fieldname + "\"]"
                                + pseudoclass)
}


function getFormFieldElemsMeta(prefix, caseid) {
  if (caseid !== null) {
    cond = "^=\"" + prefix + ":\""
  } else {
    cond = "=\"" + prefix + "\""
  }

  return document.querySelectorAll("#formularpage *[name" + cond + "]")
}


function getFormGroupElemsMeta(prefix) {
  return document.querySelectorAll("#formularpage fieldset[id^=\"" + prefix + ":\"]")
}


function getHiddenFieldStr(fieldname) {
  var elem = getFormFieldElem(fieldname)
  return elem === null ? null : elem.value
}


function splitIdentifier(identifier) {
  var parts = identifier.split(":")
  if (parts.length === 1) {
    return [parts[0], null, null]
  } else if (parts.length === 2) {
    return [parts[0], null, parts[1]]
  } else if (parts.length === 3) {
    return parts
  } else {
    console.log("splitIdentifier: unexpected number of parts: " + parts)
    return null
  }
}


function rgIdFromIdentifier(identifier) {
  return splitIdentifier(identifier)[1]
}


function parseDate(datestr) {
  var parts = datestr.split(".")
  if (parts.length === 3) {
    // Date takes the as a monthIndex where January is 0 and December is
    // 11, so we have to convert the month value.
    return new Date(parseInt(parts[2], 10), // year
                    parseInt(parts[1], 10) - 1, // month
                    parseInt(parts[0], 10)) // day
  }
  return null
}

function getFieldValue(fieldname, caseid, rgid) {
  fieldInfo = form_info.fields[fieldname]

  var fullName = fieldname
  if (fieldInfo.rgs.length > 0) {
    if (rgid !== null) {
      fullName += (rgid !== null ? ":" + rgid : "")
    } else {
      console.log("getFieldValue: no rgid for field in RG: " + fieldname)
    }
  }
  if (caseid !== null) {
    fullName += ":" + caseid
  }

  var converter = fieldInfo.converter
  var elem = getFormFieldElem(fullName, converter === "radio" ? ":checked" : "")
  if (elem !== null) {
    var value = null
    switch (converter) {
    case "int":
    case "radio":
      value = parseInt(elem.value, 10)
      if (isNaN(value)) {
        // The value could not be parsed as an int, possibly because the
        // string is empty.
        value = null;
      }
      break;
    case "bool":
      value = elem.checked ? 1 : 0
      break;
    case "str":
      // nothing to be done
      value = elem.value
      break;
    case "date":
      value = parseDate(elem.value)
      break;
    case "choice":
      // We need to distinguish the following cases:
      //   1. elem is a select element.
      //      If the element allows multiselect the value is an array
      //      containing all the selected options as ints. Otherwise the
      //      value is a single int with the selected option.
      //   2. elem is a checkbox
      //      In this case it's only one of the checkboxes, so we do a new
      //      search for all the relevant checkboxes and the value is an
      //      array containing the values of each checked checkbox as
      //      ints.
      if (elem.multiple) {
        value = []
        var selected = elem.selectedOptions
        for (var i = 0; i < selected.length; i++) {
          value.push(parseInt(selected[i].value, 10))
        }
      } else if (elem.type === "checkbox") {
        var checkboxes = document.querySelectorAll("#formularpage *[name=\""
                                                   + fullName + "\"]")
        var value = []
        for (var i = 0; i < checkboxes.length; i++) {
          if (checkboxes[i].checked) {
            value.push(parseInt(checkboxes[i].value, 10))
          }
        }
      } else {
        value = parseInt(elem.value, 10)
      }
      break;
    default:
      console.log("Unknown converter: " + converter)
    }
    return value
  } else {
    console.log("getFieldValue: cannot find field " + fullName)
  }
}


function isKnownValue(x) {
  switch (typeof x) {
  case "string":
    return x !== "unbekannt"
  case "number":
    return x !== -9999999
  case "object":
    if (x instanceof Date) {
      return +x !== +(new Date(1, 1, 1))
    }
    return true
  default:
    return true
  }
}

function evalExpr(expr, caseid, rgid) {
  var op = expr[0]
  switch (op) {
  case "And":
    return evalExpr(expr[1], caseid, rgid) && evalExpr(expr[2], caseid, rgid)
  case "Or":
    return evalExpr(expr[1], caseid, rgid) || evalExpr(expr[2], caseid, rgid)
  case "EQ":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    // FIXME: this comparison does not work correctly with Date values
    return a === b
  case "NE":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    // FIXME: this comparison does not work correctly with Date values
    return a !== b
  case "GT":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    return a > b
  case "GE":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    return a >= b
  case "LT":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    return a < b
  case "LE":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    return a <= b
  case "CONTAINS":
    var a = evalExpr(expr[1], caseid, rgid)
    var values = evalExpr(expr[2], caseid, rgid)
    return values.indexOf(a) >= 0
  case "NOT":
    var a = evalExpr(expr[1], caseid, rgid)
    return !a
  case "ISSET":
    var a = evalExpr(expr[1], caseid, rgid)
    return a !== null
  case "KNOWN":
    var a = evalExpr(expr[1], caseid, rgid)
    return isKnownValue(a)
  case "Const":
    return expr[1]
  case "List":
    return expr.slice(1).map(function (e) { return evalExpr(e, caseid, rgid) })
  case "Date":
    var day = evalExpr(expr[1], caseid, rgid)
    var month = evalExpr(expr[2], caseid, rgid)
    var year = evalExpr(expr[3], caseid, rgid)
    return new Date(year, month, day)
  case "Field":
    return getFieldValue(expr[1], caseid, rgid)
  default:
    console.log("evalExpr: could not handle expr " + expr)
  }
}


function evaluateConditionals(names, conditionals, caseid, rgid) {
  var total = true
  for (var i = 0; i < names.length; i++) {
    var condInfo = conditionals[names[i]]
    var result = evalExpr(condInfo.expr, caseid, rgid)

    total = total && result
  }
  return total
}


function containingFieldSet(elem) {
  while (elem !== null && elem.tagName !== "FIELDSET") {
    elem = elem.parentElement
  }
  return elem
}


function showHideWarning(elem, doShow) {
  // Determine the name used in the li-element in the warning box that
  // show the warning for elem.
  //
  // For most input types, this is the name of the element. For
  // bool-groups, the warning in the warning box is linked to the
  // containing fieldset element, we need to use the fieldset's id as
  // the warning_elem_name
  var warning_elem_name = elem.name
  var fieldset = containingFieldSet(elem)
  if (fieldset !== null && fieldset.id.match("^bool-group-")) {
    warning_elem_name = fieldset.id
  }

  var warning = document.querySelector('li[id^="formed-warning"]'
                                       + '[id$="' + warning_elem_name + '"]')
  if (warning !== null) {
    if (doShow) {
      warning.removeAttribute("style")
    } else {
      warning.style = "display:none;"
    }
    return true
  }
  return false
}

function showHideWarningBox() {
  var box = document.querySelector('div.alert-warning')
  if (box !== null) {
    var visibleDescendants = box.querySelectorAll('li:not([style])')
    if (visibleDescendants.length !== 0) {
      box.removeAttribute("style")
    } else {
      box.style = "display:none;"
    }
  }
}


function updateWarningClass(elem, add) {
  if (elem !== null) {
    if (add) {
      if (!elem.classList.contains("error")) {
        elem.classList.add("warning")
      }
    } else {
      elem.classList.remove("warning")
    }
  }
}

function updateFormItemClass(elem) {
  var add = !elem.disabled
  updateWarningClass(elem, add)
}


function recomputeConditionals(event) {
  var identifier = splitIdentifier(getHiddenFieldStr("page"))
  var pagename = identifier[0]
  var pagergid = identifier[1]
  var caseid   = identifier[2]
  var conditionals = form_info.pages[pagename].conds

  var affected = form_info.pages[pagename].affected_fields
  for (var fieldName in affected) {
    var fieldConds = affected[fieldName]
    var fieldElems = getFormFieldElemsMeta(fieldName, caseid)
    for (var elemIdx = 0; elemIdx < fieldElems.length; elemIdx++) {
      var elem = fieldElems[elemIdx]
      var rgid = rgIdFromIdentifier(elem.name)
      var result = evaluateConditionals(fieldConds, conditionals, caseid, rgid)

      elem.disabled = !result

      var hasWarnings = showHideWarning(elem, result)
      if (hasWarnings) {
        updateFormItemClass(elem)
      }
    }
  }

  affected = form_info.pages[pagename].affected_groups
  for (var groupName in affected) {
    var groupConds = affected[groupName]
    var groupElems = getFormGroupElemsMeta(groupName)
    for (var elemIdx = 0; elemIdx < groupElems.length; elemIdx++) {
      // The elements found by getFormGroupElemsMeta are the fieldsets
      // that correspond to the groups and we need to use their IDs to
      // extract RG-IDs. The element we need to show/hide is the parent
      // of that fieldset, though.
      var fieldSet = groupElems[elemIdx]
      var parentDiv = fieldSet.parentElement
      var rgid = rgIdFromIdentifier(fieldSet.id)
      var result = evaluateConditionals(groupConds, conditionals, caseid, rgid)

      if (result) {
        parentDiv.classList.remove("invisible_conditional")
      } else {
        parentDiv.classList.add("invisible_conditional")
      }
    }
  }

  showHideWarningBox()

  startNoticeRuleTimeout(event)
}

var noticeRuleTimeoutId = null
var pendingNoticeFields = new Set()

function startNoticeRuleTimeout(event) {
  if (noticeRuleTimeoutId !== null) {
    clearTimeout(noticeRuleTimeoutId)
  }
  pendingNoticeFields.add(event.target.name)
  noticeRuleTimeoutId = setTimeout(handlePendingNoticeRules, 500)
}

function handlePendingNoticeRules() {
  var identifier = splitIdentifier(getHiddenFieldStr("page"))
  var pagename = identifier[0]
  var caseid   = identifier[2]
  var rules = form_info.pages[pagename].rules

  pendingNoticeFields.forEach((fullFieldName) => {
    var identifier = splitIdentifier(fullFieldName)
    var fieldName = identifier[0]
    var rgid = rgIdFromIdentifier(fullFieldName)
    for (var ruleName in rules) {
      var rule = rules[ruleName]
      if (fieldName === rule.marked) {
        if (!evalExpr(rule.expr, caseid, rgid)) {
          alert(rule.text)
        }
      }
    }
  })
  pendingNoticeFields.clear()
}

function installChangeHandler(elem) {
  elem.addEventListener("input", function(event) {
    recomputeConditionals(event)
  })
}


window.addEventListener("load", function(event) {
  var page = getHiddenFieldStr("page")
  if (page === null) {
    // don't do anything on pages that aren't forms
    return
  }

  var identifier = splitIdentifier(page)
  var pagename = identifier[0]
  var pagergid = identifier[1]
  var caseid   = identifier[2]
  var sensitive = form_info.pages[pagename].sensitive

  for (var i = 0; i < sensitive.length; i++) {
    var metaName = sensitive[i]
    var elems = getFormFieldElemsMeta(metaName, caseid)
    for (var j = 0; j < elems.length; j++) {
      installChangeHandler(elems[j])
    }
  }
});
