// Completion for postal code and town fields
//
// This code assumes that two variables have been set by other means
// e.g. with script-elements in templates:
//
// plz_pairs
//
//     Array of pairs of postal code and town field names in that order,
//     e.g.
//
//     [["addresse_plz", "addresse_ort"], ["another_plz", "another_ort"]]
//
//
// plz_table
//
//     Array of postal code/town name pairs, e.g.
//
//     [["12345", "Somecity"],
//      ["23456", "Elsewhere"]]

function updateOtherField(elem, value, otherField) {
    var plzField;
    var ortField;
    var parts = elem.id.split(":");
    parts[0] = otherField;
    if (/plz/.test(elem.id)){
        plzField = $(elem);
        ortField= $("#" + parts.join("\\:"));
    } else {
        plzField = $("#" + parts.join("\\:"));
        ortField= $(elem);
    }

    if (plzField.val().length !== 5) {
        plzField.val(value)
    }
    if (!ortField.val().trim()) {
        ortField.val(value);
    }
}


function installChooser(thisField, thisKey, otherField, otherKey, search) {
    $("input[name='" + thisField + "'], input[name^='" + thisField + ":']").each(function () {
        $(this).typeahead(null, {
            name: 'plz-ort-search',
            source: search.ttAdapter(),
            templates: {
                suggestion: function (d) {
                    return `<div style="width:15rem">${d.plz} ${d.ort}</div>`;
                },
            },
            displayKey: thisKey,
        }).on("typeahead:selected", function (event, suggestion, name) {
            updateOtherField(event.target, suggestion[otherKey], otherField);
        });
        $(this).addClass("typeahead");
    })
}


$(document).ready(function() {
    var plzObjects = plz_table.map(function(ortPLZTuple){
        return {plz: ortPLZTuple[1], ort: ortPLZTuple[0]}
    });

    var plzSearch = new Bloodhound({
        datumTokenizer: Bloodhound.tokenizers.obj.whitespace("plz"),
        queryTokenizer: Bloodhound.tokenizers.whitespace,
        limit: 15,
        local: plzObjects,
    });
    plzSearch.initialize();

    var ortSearch = new Bloodhound({
        datumTokenizer: Bloodhound.tokenizers.obj.whitespace("ort"),
        queryTokenizer: Bloodhound.tokenizers.whitespace,
        limit: 20,
        local: plzObjects,
    });
    ortSearch.initialize();

    for (i = 0; i < plz_pairs.length; i++) {
        var plzField = plz_pairs[i][0];
        var ortField = plz_pairs[i][1];
        installChooser(plzField, "plz", ortField, "ort", plzSearch);
        installChooser(ortField, "ort", plzField, "plz", ortSearch);
    }
});
