# -*- coding: utf-8 -*-
import unittest

from mpulsweb.lib.helper.filters import html_shorten

class HTMLShortenTestCase(unittest.TestCase):

    def test_no_short(self):
        self.assertEqual(html_shorten(u'<strong>foo bar</strong>', 7),
                         '<p><strong>foo bar</strong></p>')

    def test_short_strong(self):
        self.assertEqual(html_shorten(u'<strong>foo bar</strong>', 6),
                         '<p><strong>foo</strong>...</p>')

    def test_short(self):
        self.assertEqual(html_shorten(u'<strong>foo bar<em>bar</em></strong>', 6),
                         '<p><strong>foo</strong>...</p>')

    def test_entitycode(self):
        self.assertEqual(html_shorten(u'&lt;&gt;&amp;'), '<p>&lt;&gt;&amp;</p>')

    def test_charref(self):
        self.assertEqual(html_shorten(u'&lt;&#62;&#x26;'),
                         '<p>&lt;&#62;&#x26;</p>')

    def test_missing_end(self):
        self.assertEqual(html_shorten(u'<strong>foo'),
                         '<p><strong>foo</strong></p>')

    def test_wrong_order(self):
         self.assertEqual(html_shorten(u'<strong><em>foo</strong></em>'), 'foo\n')

    def test_wrong_order_more(self):
         self.assertEqual(html_shorten(u'<strong><em>foo</strong></em> more'), 'foo more\n')

    def test_openclosed_p(self):
        self.assertEqual(html_shorten('<strong/><strong>foo bar</strong>'),
                         '<p><strong>foo bar</strong></p>')

    def test_umlauts(self):
        self.assertEqual(html_shorten(u'testüäöÜÄÖ'),
                         u'<p>testüäöÜÄÖ</p>')

    def test_empty_string(self):
        self.assertEqual(html_shorten(''), '')

    def test_br(self):
        self.assertEqual(html_shorten('Absatz 1<br> Absatz 2'),
                         '<p>Absatz 1 Absatz 2</p>')

    def test_br_openclose(self):
        self.assertEqual(html_shorten('Absatz 1<br/> Absatz 2'),
                         '<p>Absatz 1 Absatz 2</p>')


if __name__ == "__main__":
    unittest.main()
