SET ROLE :adm_ka_owner;
BEGIN;

--
-- Views auf dem Logbuch
--

CREATE OR REPLACE VIEW ka_logbuch_tbl_view
    AS SELECT
        ka_logbuch_tbl.id         AS id,
        ka_logbuch_tbl.uuid_id    AS uuid_id,
        ka_logbuch_tbl.master_id  AS master_id,
        ka_logbuch_tbl.datum      AS datum,
        ka_logbuch_tbl.bearbeiter AS bearbeiter,
        ARRAY(SELECT choice_id
                FROM nm_logbuch_art_tbl nm
               WHERE nm.parent_id = ka_logbuch_tbl.id) AS art,
        ka_logbuch_tbl.typ        AS typ,
        ka_logbuch_tbl.kurz_notiz AS kurz_notiz,
        ka_logbuch_tbl.notiz      AS notiz,
        ka_logbuch_tbl.dauer      AS dauer
    FROM ka_logbuch_tbl
    INNER JOIN master_tbl ON
        ka_logbuch_tbl.master_id = master_tbl.id
    WHERE
        master_tbl.bearbeiter_id = getuserid()
        OR master_tbl.id IN (
            SELECT nm.master_id FROM nm_benutzer_master_tbl nm
            WHERE nm.benutzer_id IN (
                SELECT nm2.gruppe_id FROM nm_benutzer_gruppe_tbl nm2
                WHERE nm2.benutzer_id = getuserid())
        )
        OR pg_has_role('admin','MEMBER');




CREATE OR REPLACE RULE ka_logbuch_update_rule
    AS ON UPDATE TO ka_logbuch_tbl_view
    DO INSTEAD (
        UPDATE ka_logbuch_tbl SET
            datum      = new.datum,
            bearbeiter = new.bearbeiter,
            typ        = new.typ,
            kurz_notiz = new.kurz_notiz,
            notiz      = new.notiz,
            dauer      = new.dauer
        WHERE ka_logbuch_tbl.id = new.id;
        DELETE FROM nm_logbuch_art_tbl WHERE parent_id = new.id;
        INSERT INTO nm_logbuch_art_tbl (parent_id, choice_id)
             SELECT new.id as parent_id, a.id as choice_id
               FROM ka_logbuch_art_tbl a
              WHERE a.id = ANY (new.art);
    );

CREATE OR REPLACE RULE ka_logbuch_delete_rule
    AS ON DELETE TO  ka_logbuch_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_logbuch_insert_rule
    AS ON INSERT TO ka_logbuch_tbl_view DO INSTEAD NOTHING;

GRANT SELECT ON ka_logbuch_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON ka_logbuch_tbl_view TO GROUP :cm_group;
GRANT SELECT ON ka_logbuch_tbl_view TO GROUP :admin_group;


CREATE OR REPLACE VIEW nm_logbuch_art_tbl_view
    AS SELECT parent_id,
              choice_id
	 FROM nm_logbuch_art_tbl nm
	 JOIN ka_logbuch_tbl_view l ON l.id = nm.parent_id;

GRANT SELECT ON nm_logbuch_art_tbl_view TO GROUP :cm_group;
GRANT SELECT ON nm_logbuch_art_tbl_view TO GROUP :admin_group;


CREATE OR REPLACE VIEW ka_logbuch_dokumente_tbl_view
    AS SELECT ka_logbuch_dokumente_tbl.id,
              ka_logbuch_dokumente_tbl.uuid_id,
              ka_logbuch_tbl.master_id,
              ka_logbuch_dokumente_tbl.logbuch_id,
              ka_logbuch_dokumente_tbl.groesse,
              ka_logbuch_dokumente_tbl.mime,
              ka_logbuch_dokumente_tbl.daten,
              ka_logbuch_dokumente_tbl.name,
              ka_logbuch_dokumente_tbl.datum
         FROM ka_logbuch_dokumente_tbl
         JOIN ka_logbuch_tbl
           ON ka_logbuch_dokumente_tbl.logbuch_id = ka_logbuch_tbl.id
         JOIN master_tbl ON ka_logbuch_tbl.master_id = master_tbl.id
        WHERE master_tbl.bearbeiter_id = getuserid()
           OR master_tbl.id
              IN (SELECT nm.master_id
                    FROM nm_benutzer_master_tbl nm
                   WHERE nm.benutzer_id
                         IN (SELECT nm2.gruppe_id
                               FROM nm_benutzer_gruppe_tbl nm2
                              WHERE nm2.benutzer_id = getuserid()))
           OR pg_has_role('admin','MEMBER');

CREATE OR REPLACE RULE ka_logbuch_dokumente_delete_rule
    AS ON DELETE TO ka_logbuch_dokumente_tbl_view
       DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_logbuch_dokumente_insert_rule
    AS ON INSERT TO ka_logbuch_dokumente_tbl_view
       DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_logbuch_dokumente_update_rule
    AS ON UPDATE TO ka_logbuch_dokumente_tbl_view
       DO INSTEAD
          UPDATE ka_logbuch_dokumente_tbl
             SET groesse = new.groesse,
                 mime = new.mime,
                 daten = new.daten,
                 name = new.name,
                 datum = new.datum
           WHERE ka_logbuch_dokumente_tbl.id = new.id;

GRANT SELECT ON ka_logbuch_dokumente_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON ka_logbuch_dokumente_tbl_view TO GROUP :cm_group;
GRANT SELECT ON ka_logbuch_dokumente_tbl_view TO GROUP :admin_group;


COMMIT;
