#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""Merge an XLS-file generated with gettext-to-xlwt.py into a po-file.
Usage:
gettext-from-xlrd.py  XLS_FILE  PO_FILE

The contents of PO_FILE are updated with the contents of the XLS_FILE.
This only updates the values of keys in PO_FILE with the corresponding
values from the XLS_FILE. No new entries in PO_FILE will be produced,
"""

import sys
import optparse

from xlrd import open_workbook, XL_CELL_TEXT

from babel.messages.pofile import read_po, write_po
from babel.messages import Catalog


def read_xls(filename, key_column, msg_column):
    messages = {}
    book = open_workbook(filename)
    for sheet in book.sheets():
        for rowidx in range(1, sheet.nrows):
            keycell = sheet.cell(rowidx, key_column)
            msgcell = sheet.cell(rowidx, msg_column)
            if keycell.ctype == XL_CELL_TEXT and msgcell.ctype == XL_CELL_TEXT:
                messages[keycell.value] = msgcell.value
    return messages


def merge_from_xls(pofile, xlsfile, key_column, msg_column):
    with open(pofile) as r:
        catalog = read_po(r)

    new_messages = read_xls(xlsfile, key_column, msg_column)

    for msg in list(catalog):
        if msg.id == "":
            continue

        # NOTE: Reading the docs of babel, it might seem better to use
        # the catalog's add method to update the messages, but due to a
        # bug in babel 0.9.6 (and maybe other versions) that doesn't
        # actually update the message string. So we directly modify the
        # msg objects in place, which does work.
        if msg.pluralizable:
            msg.string = (new_messages.get(msg.id, ""),
                          new_messages.get(msg.id, ""))
        else:
            msg.string = new_messages.get(msg.id, "")


    with open(pofile, "w") as w:
        write_po(w, catalog)


def main():
    usage = "Usage: %prog [options] PO-FILE XLS-FILE"
    parser = optparse.OptionParser(usage=usage)
    parser.add_option("--key-column", type="int", default=0,
                      help="Number of the column containing the keys.")
    parser.add_option("--msg-column", type="int", default=3,
                      help=("Number of the column containing the translated"
                            " message."))
    options, args = parser.parse_args()

    if len(args) != 2:
        print >>sys.stderr, ("The po-file and the xls-file must be given"
                             " as command line arguments")

    merge_from_xls(args[0], args[1], options.key_column, options.msg_column)


main()
